<?php
/**
* Data Transaction Class
* A data abstraction class used to handle all database transactions. This class is a controller to the data object class it uses.
* @package		phpOpenFW
* @subpackage 	Database_Tools
* @author 		Christian J. Clark
* @copyright	Copyright (c) Christian J. Clark
* @license		http://www.gnu.org/licenses/gpl-2.0.txt
* @version 		Started: 9-20-2005 updated: 3-24-2010
*/

//***************************************************************
/**
 * Data Transaction Class
 * @package		phpOpenFW
 * @subpackage	Database_Tools
 */
//***************************************************************
class data_trans {

	//************************************************************************	
	// Class variables
	//************************************************************************
	/**
	* @var string data source type (mysql, mysqli, pgsql, ldap, oracle, mssql)
	**/
	private $data_type;
	
	/**
	* @var bool Print the queries run through this transaction (Yes or No)
	**/
	private $print_query;
	
	/**
	* @var Object Internal Data Object 
	**/
	private $data_object;

	/**
	* Constructor function
	*
	* Initializes data transaction, setting all necessary variables from specified data source
	*
	* @param string The name of data source to use as specified in config.inc.php
	**/
	//*************************************************************************
	// Constructor function
	//*************************************************************************
	public function __construct($data_src='', $tmp_trans_type = 'qry')
	{
		// Data Source
		if ($data_src != '') {
			if (!isset($_SESSION[$data_src])) {
				trigger_error("Error: [data_trans]::__construct(): Invalid Data Source '$data_src'.");
				exit(1);
			}
		}
		else {
			if (isset($_SESSION['default_data_source'])) {
				$data_src = $_SESSION['default_data_source'];
			}
			else {
				trigger_error('Error: [data_trans]::__construct(): Data Source not given and default data source is not set.');
				exit(2);
			}
		}
		
        $this->data_type = $_SESSION[$data_src]['type'];
	
        // Data Library
        $data_lib = dirname(__FILE__) . '/lib/';
        
        // Include (dt_structure) class
        $data_structure_lib = $data_lib . 'dt_structure.class.php';
        
        if (file_exists($data_structure_lib)) {
            include_once($data_structure_lib);
        }
        else {
            trigger_error("Error: $data_structure_lib does not exist!!\n");
        }
        
        // Include necessary Data Object Library
        $data_object_lib = $data_lib . 'dt_' . $this->data_type . '.class.php';
        
        if (file_exists($data_object_lib)) {
            include_once($data_object_lib);
        }
        else {
            trigger_error("Error: $data_object_lib does not exist!!\n");
        }
			
        // Create new Data Object
        $new_obj_code = "\$this->data_object = new dt_" . $this->data_type . "(\$data_src, \$tmp_trans_type); return \$this->data_object;";
        $this->data_object = eval($new_obj_code);
	}

	//*************************************************************************
	/**
	* Destructor Function
	**/
	//*************************************************************************
    public function __destruct() {}

	/**
	* Opens a connection to the specified data source based on the data source type
	**/
	//*************************************************************************
	// Make a connection to the given source and store the handle
	//*************************************************************************
	private function data_open() { $this->data_object->open(); }

	/**
	* Closes a connection to the specified data source based on the data source type
	**/
	//*************************************************************************
	// Close the connection to the data source
	//*************************************************************************
	private function data_close() { $this->data_object->close(); }

	/**
	* Executes a query based on the data source type
	* @param mixed MySQL: SQL Statement, PostgreSQL: SQL Statement, LDAP: properly formatted and filled array
	* @param string 'anon' - Anonymous Bind, 'user' - User Bind, 'admin' - Admin Bind
	**/
	//*************************************************************************
	// Execute a query and store the record set
	//*************************************************************************
	public function data_query($query, $bind='anon')
	{
		// Unset last_id
		$this->data_object->clear_last_insert_id();
		
		// Clear results
		$this->data_object->clear_result();
		
		// Open the Data Connection
		$this->data_open();
		
		// Bind if necessary
		switch (strtolower($bind)) {
			case 'admin':
				$this->data_admin_bind();
				break;
				
			case 'user':
				$this->data_user_bind($_SESSION['ldap_userid'], $_SESSION['passwd']);
				break;
		}

		// Print Query (if necessary)
		if ($this->print_query) {
			if (is_array($query)) {
                print "<pre>\n";
                print_r($query);
                print "</pre>\n";
			}
			else { print $query . "<br/>\n"; }
		}

		// Execute the Query
		$this->data_object->query($query);
		
		// Close the Data Connection
		$this->data_close();
	}

	/**
	* Bind to a Data Source (Used for Authentication Primarily)
	* @param string userid of the user trying to bind to data source
	* @param string password of the user trying to bind to data source
	**/
	//*************************************************************************
	// Bind to a data source as a specified user
	// *** Used for Authentication
	//*************************************************************************
	private function data_bind($user, $pass)
	{
		return $this->data_object->bind($user, $pass);
	}

	/**
	* Bind to the Data Source with user and password specified in the configuration
	**/
	//*************************************************************************
	// Bind to a data source as the configured user
	//*************************************************************************
	public function data_admin_bind()
	{
		return $this->data_bind($this->data_object->get_user(), $this->data_object->get_pass());
	}
	
	/**
	* Bind to the Data Source with the provided user and password
	**/
	//*************************************************************************
	// Bind to a data source as a specified user
	//*************************************************************************
	public function data_user_bind($user, $pass)
	{
		return $this->data_bind($user, $pass);
	}
	
	/**
	* Return the current record set in the form of an associative array
	* @return array current record set in the form of an associate array 
	**/
	//*************************************************************************
	// Extract the record set to local variables
	//*************************************************************************
	public function data_assoc_result()
	{
		return $this->data_object->assoc_result();
	}
	
	/**
	* Return an abbreviated form of the current record set in the form of an associative array
	* @param string field to be used as the 'key' in the associative array
	* @param string field to be used as the 'value' in the associative array
	* @return array abbreviated form of the current record set in the form of an associative array 
	**/
	//*************************************************************************
	// Extract an abbreviated record set to a 'key' => 'value' array
	//*************************************************************************
	public function data_key_val($key, $value)
	{
		return $this->data_object->key_val_result($key, $value);
	}

	//*************************************************************************
	/**
	* Return the current record set in the form of an associative array with $key as the key for each record
	* @param string The index of the field to be used as the 'key' for each record
	* @return array Return the current record set in the form of an associative array with $key as the key for each record
	**/
	//*************************************************************************
	public function data_key_assoc($key)
	{
		return $this->data_object->key_assoc_result($key);
	}

	/**
	* Return ID of the last inserted row for the current session
	* @return integer ID of the last inserted row for the current session
	**/
	//*************************************************************************
	// Return the ID of the last inserted row
	//*************************************************************************
	public function last_insert_id()
	{
		return $this->data_object->last_insert_id();
	}

	/**
	* Set the current transaction type
	* @param string Query Type: '?' - query (default, all SQL queries), '+' - add (LDAP), '~' - modify (LDAP), 
	* '-' - delete (LDAP), '?1' - one level query (LDAP)
	**/
	//*************************************************************************
	// Set the Transaction type
	// '?' - query (default), '+' - add, '~' - modify, '-' - delete
	// '?1' - one level query (LDAP)
	//*************************************************************************
	public function set_trans_type($in_type)
	{
		switch ($in_type) {
			case '+':
				$tmp_trans_type = 'add';
				break;
				
			case '~':
				$tmp_trans_type = 'mod';
				break;
				
			case '-':
				$tmp_trans_type = 'del';
				break;
				
			case '?1':
				$tmp_trans_type = 'qry1';
				break;
					
			default:
				$tmp_trans_type = 'qry';
				break;
		}
		$this->data_object->set_trans_type($tmp_trans_type);
	}
	
	//*************************************************************************
	/**
	* Return the number rows in the current record set
	* @return the number rows in the current record set
	**/
	//*************************************************************************
	public function data_num_rows()
	{
		return $this->data_object->num_rows();
	}

	//*************************************************************************
	/**
	* Show the current record set in raw format
	**/
	//*************************************************************************	
	public function data_raw_output()
	{
		print "<pre>\n";
		print_r($this->data_object->assoc_result());
		print "</pre>\n";
	}

	//*************************************************************************
	/**
	* Print all queries run through this transaction
	**/
	//*************************************************************************
	public function data_debug() { $this->print_query = true; }

	//*************************************************************************
	/**
	* Return whether the current user is bound to the current data source
	* @return whether the current user is bound to the current data source
	**/
	//*************************************************************************
	public function is_bound() { return $this->data_object->is_bound(); }

	//*************************************************************************
	/**
	* Prepares an SQL statement to be executed
	**/
	//*************************************************************************
	public function prepare($statement=false)
	{
		if (!$statement) {
			trigger_error('Error: [data_trans]::prepare(): No statement or invalid statement passed.');
			return false;
		}
		else {
			return $this->data_object->prepare($statement);
		}
	}

	//*************************************************************************
	/**
	* Begin a Transaction
	**/
	//*************************************************************************
	public function begin_transaction() { return $this->data_object->begin_transaction(); }

	//*************************************************************************
	/**
	* Commit current Outstanding Statements / Transaction(s)
	**/
	//*************************************************************************
	public function commit() { return $this->data_object->commit(); }

	//*************************************************************************
	/**
	* Rollback current Outstanding Statements / Transaction(s)
	**/
	//*************************************************************************
	public function rollback() { return $this->data_object->rollback(); }

}

?>